@@ -1,6 +1,6 @@
 import warnings
 from types import ModuleType
-from typing import Any, Dict, List, Optional, Union, overload
+from typing import Any, Optional, Union, overload
 
 import pandas as pd
 
@@ -36,7 +36,7 @@ class Registry:
         *,
         database_name: Optional[str] = None,
         schema_name: Optional[str] = None,
-        options: Optional[Dict[str, Any]] = None,
+        options: Optional[dict[str, Any]] = None,
     ) -> None:
         """Opens a registry within a pre-created Snowflake schema.
 
@@ -75,7 +75,9 @@ class Registry:
             )
 
         self._model_manager = model_manager.ModelManager(
-            session, database_name=self._database_name, schema_name=self._schema_name
+            session,
+            database_name=self._database_name,
+            schema_name=self._schema_name,
         )
 
         self.enable_monitoring = options.get("enable_monitoring", True) if options else True
@@ -105,16 +107,19 @@ class Registry:
         model_name: str,
         version_name: Optional[str] = None,
         comment: Optional[str] = None,
-        metrics: Optional[Dict[str, Any]] = None,
-        conda_dependencies: Optional[List[str]] = None,
-        pip_requirements: Optional[List[str]] = None,
-        artifact_repository_map: Optional[Dict[str, str]] = None,
-        target_platforms: Optional[List[model_types.SupportedTargetPlatformType]] = None,
+        metrics: Optional[dict[str, Any]] = None,
+        conda_dependencies: Optional[list[str]] = None,
+        pip_requirements: Optional[list[str]] = None,
+        artifact_repository_map: Optional[dict[str, str]] = None,
+        resource_constraint: Optional[dict[str, str]] = None,
+        target_platforms: Optional[list[model_types.SupportedTargetPlatformType]] = None,
         python_version: Optional[str] = None,
-        signatures: Optional[Dict[str, model_signature.ModelSignature]] = None,
+        signatures: Optional[dict[str, model_signature.ModelSignature]] = None,
         sample_input_data: Optional[model_types.SupportedDataType] = None,
-        code_paths: Optional[List[str]] = None,
-        ext_modules: Optional[List[ModuleType]] = None,
+        user_files: Optional[dict[str, list[str]]] = None,
+        code_paths: Optional[list[str]] = None,
+        ext_modules: Optional[list[ModuleType]] = None,
+        task: model_types.Task = model_types.Task.UNKNOWN,
         options: Optional[model_types.ModelSaveOption] = None,
     ) -> ModelVersion:
         """
@@ -148,6 +153,7 @@ class Registry:
                 Format: {channel_name: artifact_repository_name}, where:
                    - channel_name: The name of the Conda package channel (e.g., 'condaforge') or 'pip' for pip packages.
                    - artifact_repository_name: The name or URL of the repository to fetch packages from.
+            resource_constraint: Mapping of resource constraint keys and values, e.g. {"architecture": "x86"}.
             target_platforms: List of target platforms to run the model. The only acceptable inputs are a combination of
                 {"WAREHOUSE", "SNOWPARK_CONTAINER_SERVICES"}. Defaults to None.
             python_version: Python version in which the model is run. Defaults to None.
@@ -156,10 +162,15 @@ class Registry:
                 infer the signature. If not None, sample_input_data should not be specified. Defaults to None.
             sample_input_data: Sample input data to infer model signatures from.
                 It would also be used as background data in explanation and to capture data lineage. Defaults to None.
+            user_files: Dictionary where the keys are subdirectories, and values are lists of local file name
+                strings. The local file name strings can include wildcards (? or *) for matching multiple files.
             code_paths: List of directories containing code to import. Defaults to None.
             ext_modules: List of external modules to pickle with the model object.
                 Only supported when logging the following types of model:
                 Scikit-learn, Snowpark ML, PyTorch, TorchScript and Custom Model. Defaults to None.
+            task: The task of the Model Version. It is an enum class Task with values TABULAR_REGRESSION,
+                TABULAR_BINARY_CLASSIFICATION, TABULAR_MULTI_CLASSIFICATION, TABULAR_RANKING, or UNKNOWN. By default,
+                it is set to Task.UNKNOWN and may be overridden by inferring from the Model Object.
             options (Dict[str, Any], optional): Additional model saving options.
 
                 Model Saving Options include:
@@ -171,6 +182,10 @@ class Registry:
                     Warehouse. It detects any ==x.y.z in specifiers and replaced with >=x.y, <(x+1). Defaults to True.
                 - function_type: Set the method function type globally. To set method function types individually see
                     function_type in model_options.
+                - target_methods: List of target methods to register when logging the model.
+                  This option is not used in MLFlow models. Defaults to None, in which case the model handler's
+                  default target methods will be used.
+                - save_location: Location to save the model and metadata.
                 - method_options: Per-method saving options. This dictionary has method names as keys and dictionary
                     values with the desired options.
 
@@ -219,6 +234,7 @@ class Registry:
             "conda_dependencies",
             "pip_requirements",
             "artifact_repository_map",
+            "resource_constraint",
             "target_platforms",
             "python_version",
             "signatures",
@@ -231,17 +247,18 @@ class Registry:
         model_name: str,
         version_name: Optional[str] = None,
         comment: Optional[str] = None,
-        metrics: Optional[Dict[str, Any]] = None,
-        conda_dependencies: Optional[List[str]] = None,
-        pip_requirements: Optional[List[str]] = None,
-        artifact_repository_map: Optional[Dict[str, str]] = None,
-        target_platforms: Optional[List[model_types.SupportedTargetPlatformType]] = None,
+        metrics: Optional[dict[str, Any]] = None,
+        conda_dependencies: Optional[list[str]] = None,
+        pip_requirements: Optional[list[str]] = None,
+        artifact_repository_map: Optional[dict[str, str]] = None,
+        resource_constraint: Optional[dict[str, str]] = None,
+        target_platforms: Optional[list[model_types.SupportedTargetPlatformType]] = None,
         python_version: Optional[str] = None,
-        signatures: Optional[Dict[str, model_signature.ModelSignature]] = None,
+        signatures: Optional[dict[str, model_signature.ModelSignature]] = None,
         sample_input_data: Optional[model_types.SupportedDataType] = None,
-        user_files: Optional[Dict[str, List[str]]] = None,
-        code_paths: Optional[List[str]] = None,
-        ext_modules: Optional[List[ModuleType]] = None,
+        user_files: Optional[dict[str, list[str]]] = None,
+        code_paths: Optional[list[str]] = None,
+        ext_modules: Optional[list[ModuleType]] = None,
         task: model_types.Task = model_types.Task.UNKNOWN,
         options: Optional[model_types.ModelSaveOption] = None,
     ) -> ModelVersion:
@@ -276,6 +293,7 @@ class Registry:
                 Format: {channel_name: artifact_repository_name}, where:
                    - channel_name: The name of the Conda package channel (e.g., 'condaforge') or 'pip' for pip packages.
                    - artifact_repository_name: The name or URL of the repository to fetch packages from.
+            resource_constraint: Mapping of resource constraint keys and values, e.g. {"architecture": "x86"}.
             target_platforms: List of target platforms to run the model. The only acceptable inputs are a combination of
                 {"WAREHOUSE", "SNOWPARK_CONTAINER_SERVICES"}. Defaults to None.
             python_version: Python version in which the model is run. Defaults to None.
@@ -304,6 +322,10 @@ class Registry:
                     Warehouse. It detects any ==x.y.z in specifiers and replaced with >=x.y, <(x+1). Defaults to True.
                 - function_type: Set the method function type globally. To set method function types individually see
                   function_type in model_options.
+                - target_methods: List of target methods to register when logging the model.
+                  This option is not used in MLFlow models. Defaults to None, in which case the model handler's
+                  default target methods will be used.
+                - save_location: Location to save the model and metadata.
                 - method_options: Per-method saving options. This dictionary has method names as keys and dictionary
                     values with the desired options. See the example below.
 
@@ -317,6 +339,9 @@ class Registry:
                         Defaults to None, determined automatically by Snowflake.
                     - function_type: One of supported model method function types (FUNCTION or TABLE_FUNCTION).
 
+        Raises:
+            ValueError: If extra arguments are specified ModelVersion is provided.
+
         Returns:
             ModelVersion: ModelVersion object corresponding to the model just logged.
 
@@ -339,13 +364,38 @@ class Registry:
             registry.log_model(
               model=model,
               model_name="my_model",
-              method_options=method_options,
+              options={"method_options": method_options},
             )
         """
         statement_params = telemetry.get_statement_params(
             project=_TELEMETRY_PROJECT,
             subproject=_MODEL_TELEMETRY_SUBPROJECT,
         )
+        if isinstance(model, ModelVersion):
+            # check that no arguments are provided other than the ones for copy model.
+            invalid_args = [
+                comment,
+                conda_dependencies,
+                pip_requirements,
+                artifact_repository_map,
+                resource_constraint,
+                target_platforms,
+                python_version,
+                signatures,
+                sample_input_data,
+                user_files,
+                code_paths,
+                ext_modules,
+                options,
+            ]
+            for arg in invalid_args:
+                if arg is not None:
+                    raise ValueError(
+                        "When calling log_model with a ModelVersion, only model_name and version_name may be specified."
+                    )
+            if task is not model_types.Task.UNKNOWN:
+                raise ValueError("`task` cannot be specified when calling log_model with a ModelVersion.")
+
         if pip_requirements:
             warnings.warn(
                 "Models logged specifying `pip_requirements` can not be executed "
@@ -363,6 +413,7 @@ class Registry:
             conda_dependencies=conda_dependencies,
             pip_requirements=pip_requirements,
             artifact_repository_map=artifact_repository_map,
+            resource_constraint=resource_constraint,
             target_platforms=target_platforms,
             python_version=python_version,
             signatures=signatures,
@@ -398,7 +449,7 @@ class Registry:
         project=_TELEMETRY_PROJECT,
         subproject=_MODEL_TELEMETRY_SUBPROJECT,
     )
-    def models(self) -> List[Model]:
+    def models(self) -> list[Model]:
         """Get all models in the schema where the registry is opened.
 
         Returns:
@@ -524,7 +575,7 @@ class Registry:
         subproject=telemetry.TelemetrySubProject.MONITORING.value,
     )
     @snowpark._internal.utils.private_preview(version=model_monitor_version.SNOWFLAKE_ML_MONITORING_MIN_VERSION)
-    def show_model_monitors(self) -> List[snowpark.Row]:
+    def show_model_monitors(self) -> list[snowpark.Row]:
         """Show all model monitors in the registry.
 
         Returns:
